@@ -328,12 +328,31 @@ class AsyncFleet:
         task_json = json.loads(task_string)
         return await self.load_task_from_json(task_json)
 
-    async def load_task_from_json(self, task_json: Dict) -> Task:
+    async def load_task_from_json(
+        self, task_json: Dict, raise_on_verifier_error: bool = False
+    ) -> Task:
         verifier = None
         verifier_code = task_json.get("verifier_func") or task_json.get("verifier_code")
+        verifier_sha = task_json.get("verifier_sha", "")
+
+        # Check if verifier is a nested object with code inside
+        if not verifier_code and "verifier" in task_json:
+            verifier_obj = task_json["verifier"]
+            if isinstance(verifier_obj, dict):
+                verifier_code = verifier_obj.get("code")
+                # Also extract sha256 from nested verifier if not found at top level
+                if not verifier_sha:
+                    verifier_sha = verifier_obj.get("sha256", "")
 
         # Try to find verifier_id in multiple locations
         verifier_id = task_json.get("verifier_id")
+        
+        # Check nested verifier object for verifier_id
+        if not verifier_id and "verifier" in task_json:
+            verifier_obj = task_json["verifier"]
+            if isinstance(verifier_obj, dict):
+                verifier_id = verifier_obj.get("verifier_id")
+        
         if (
             not verifier_id
             and "metadata" in task_json
@@ -353,12 +372,14 @@ class AsyncFleet:
                     verifier_id=verifier_id,
                     verifier_key=task_json.get("key", task_json.get("id")),
                     verifier_code=verifier_code,
-                    verifier_sha=task_json.get("verifier_sha", ""),
+                    verifier_sha=verifier_sha,
                 )
         except Exception as e:
-            logger.warning(
-                f"Failed to create verifier {task_json.get('key', task_json.get('id'))}: {e}"
-            )
+            error_msg = f"Failed to create verifier {task_json.get('key', task_json.get('id'))}: {e}"
+            if raise_on_verifier_error:
+                raise ValueError(error_msg) from e
+            else:
+                logger.warning(error_msg)
 
         task = Task(
             key=task_json.get("key", task_json.get("id")),
@@ -368,9 +389,13 @@ class AsyncFleet:
             ),  # Use env_id or fallback to env_key
             created_at=task_json.get("created_at"),
             version=task_json.get("version"),
+            data_id=task_json.get("data_id"),
+            data_version=task_json.get("data_version"),
             env_variables=task_json.get("env_variables", {}),
             verifier_func=verifier_code,  # Set verifier code
             verifier=verifier,  # Use created verifier or None
+            verifier_id=verifier_id,  # Set verifier_id so _rebuild_verifier works
+            verifier_sha=verifier_sha,  # Set verifier_sha
             metadata=task_json.get("metadata", {}),  # Default empty metadata
         )
         return task
@@ -382,6 +407,8 @@ class AsyncFleet:
         version: Optional[str] = None,
         team_id: Optional[str] = None,
         project_key: Optional[str] = None,
+        data_id: Optional[str] = None,
+        data_version: Optional[str] = None,
     ) -> List[Task]:
         """Load tasks for the authenticated team, with optional filtering.
 
@@ -390,6 +417,9 @@ class AsyncFleet:
             keys: Optional list of task keys to filter by
             version: Optional version to filter tasks by (client-side filter)
             team_id: Optional team_id to filter by (admin only)
+            project_key: Optional project key to filter tasks by
+            data_id: Optional data identifier to filter tasks by
+            data_version: Optional data version to filter tasks by
 
         Returns:
             List[Task] containing Task objects
@@ -403,6 +433,10 @@ class AsyncFleet:
             params["team_id"] = team_id
         if project_key is not None:
             params["project_key"] = project_key
+        if data_id is not None:
+            params["data_id"] = data_id
+        if data_version is not None:
+            params["data_version"] = data_version
 
         response = await self.client.request("GET", "/v1/tasks", params=params)
         task_list_response = TaskListResponse(**response.json())
@@ -507,6 +541,8 @@ class AsyncFleet:
                 env_id=task_response.environment_id,  # Map environment_id -> env_id
                 created_at=task_response.created_at,
                 version=task_response.version,
+                data_id=getattr(task_response, "data_id", None),  # Get data_id if available
+                data_version=getattr(task_response, "data_version", None),  # Get data_version if available
                 env_variables=task_response.env_variables or {},
                 verifier_func=verifier_func,  # Set verifier code
                 verifier=verifier,  # Use created verifier or None
@@ -517,6 +553,14 @@ class AsyncFleet:
         # Apply client-side filtering for version if specified
         if version is not None:
             tasks = [task for task in tasks if task.version == version]
+        
+        # Apply client-side filtering for data_id if specified
+        if data_id is not None:
+            tasks = [task for task in tasks if task.data_id == data_id]
+        
+        # Apply client-side filtering for data_version if specified
+        if data_version is not None:
+            tasks = [task for task in tasks if task.data_version == data_version]
 
         return tasks
 
@@ -560,40 +604,71 @@ class AsyncFleet:
             logger.info("No tasks found to export")
             return None
 
-    async def import_single_task(self, task: Task):
+    async def import_single_task(self, task: Task, project_key: Optional[str] = None):
         """Import a single task.
 
         Args:
             task: Task object to import
+            project_key: Optional project key to associate with the task
 
         Returns:
             Response from the API, or None if the import failed
         """
         try:
+            # Validate that verifier_func exists
+            if not task.verifier_func:
+                raise ValueError(
+                    f"Task {task.key} is missing verifier_func. "
+                    "All tasks must have a verifier_func to be imported."
+                )
+
+            params = {}
+            if project_key:
+                params["project_key"] = project_key
             response = await self.client.request(
-                "POST", "/v1/tasks", json=task.model_dump()
+                "POST", "/v1/tasks", json=task.model_dump(), params=params
             )
             return response
         except Exception as e:
             logger.error(f"Failed to import task {task.key}: {e}")
             return None
 
-    async def import_tasks(self, filename: str):
+    async def import_tasks(self, filename: str, project_key: Optional[str] = None):
         """Import tasks from a JSON file.
 
         Args:
             filename: Path to the JSON file of Task objects to import
+            project_key: Optional project key to associate with the tasks
 
         Returns:
             List[Task] containing imported Task objects
+
+        Raises:
+            ValueError: If any task is missing verifier_func or has invalid verifier code
         """
         with open(filename, "r", encoding="utf-8") as f:
             tasks_data = json.load(f)
 
-        # Create tasks from the loaded data
+        # Create tasks from the loaded data using load_task_from_json
+        # This will validate and create verifiers properly
         tasks = []
         for task_data in tasks_data:
-            task = Task(**task_data)
+            # Validate that verifier_func exists
+            verifier_code = task_data.get("verifier_func") or task_data.get(
+                "verifier_code"
+            )
+            if not verifier_code:
+                task_key = task_data.get("key", task_data.get("id", "unknown"))
+                raise ValueError(
+                    f"Task {task_key} is missing verifier_func. "
+                    "All tasks must have a verifier_func to be imported."
+                )
+
+            # Use load_task_from_json to properly create and validate the task
+            # Pass raise_on_verifier_error=True to fail fast on invalid verifier code
+            task = await self.load_task_from_json(
+                task_data, raise_on_verifier_error=True
+            )
             tasks.append(task)
 
         # Use semaphore to limit concurrency to 20
@@ -601,7 +676,7 @@ class AsyncFleet:
 
         async def import_with_semaphore(task):
             async with semaphore:
-                return await self.import_single_task(task)
+                return await self.import_single_task(task, project_key)
 
         # Use asyncio.gather to parallelize the imports
         responses = await asyncio.gather(
@@ -642,6 +717,29 @@ class AsyncFleet:
         )
         return TaskResponse(**response.json())
 
+    async def get_task(
+        self,
+        task_key: str,
+        version_id: Optional[str] = None,
+    ) -> TaskResponse:
+        """Get a task by key and optional version.
+
+        Args:
+            task_key: The key of the task to retrieve
+            version_id: Optional version ID to filter by
+
+        Returns:
+            TaskResponse containing the task details
+        """
+        params = {}
+        if version_id is not None:
+            params["version_id"] = version_id
+
+        response = await self.client.request(
+            "GET", f"/v1/tasks/{task_key}", params=params
+        )
+        return TaskResponse(**response.json())
+
     async def _create_verifier_from_data(
         self, verifier_id: str, verifier_key: str, verifier_code: str, verifier_sha: str
     ) -> "AsyncVerifierFunction":
